// FD1.java -- Java Screen Based Application
// Example Numerical Solution of DE by Forward Differences
//
//    by Charles W. Neville
//    copyright December 2000 by Charles W. Neville
//
// Uses command line parameters
// Sample invocation: java FD1 h
//                    where h is the increment
//
// DE is dx/dt = -2x, x(0) = 16.
// Interval is 0 <= t <= 2
//
// Compare numerical solution with exact solution x = 16exp(-2t)
//
//
// Numerical scheme is x[n+1] = x[n] + -2x[n]*h
//
// A specific example of scheme x[n+1] = x[n] + f(x[n], t[n])*h
//
// Can vary the increment h to see the effect on accuracy

public class FD1 {

   // return array of approximate numerical values of solution
   public static double[] solveIt(double h) {
      double[] x;  int N;

      N = (int)(2/h);
      x = new double[N + 1];

      x[0] = 16;  // given
      for (int n = 0; n < N; n++) {
         x[n+1] = x[n] + -2*x[n]*h;
      }

      return x;
   } // solveIt

   // display table of values of x[n] and exact values of x(t[n])
   // = 16exp(-2t[n]) for increment h.
   public static void tableDisplay(double[] x, double h) {
     double t; int N;

     N = (int)(2/h);
     // table header
     System.out.println("h:  " + h);
     System.out.println(" t   " + " x[t]  " + " 16exp(-2t) ");

     // table body
     t = 0;
     for (int n = 0; n <= N; n++) {
        System.out.println(t + " " + x[n] + " " + 16*Math.exp(-2*t));
        t = t + h;
     }
   } // tableDisplay()

   public static void main(String[] args) {

      // get value of h from first command line argument
      double h = Double.parseDouble(args[0]);

      // calculate x values by forward differences
      double[] x = solveIt(h);

      // print table of approximate values vs exact values
      // of solution
      tableDisplay(x, h);

   } // main()

} // class FD1